package com.ozacc.mail.fetch.impl;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Address;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ozacc.mail.fetch.ReceivedMail;
import com.ozacc.mail.fetch.ReceivedMail.ReceivedHeader;
import com.ozacc.mail.fetch.impl.sk_jp.AttachmentsExtractor;
import com.ozacc.mail.fetch.impl.sk_jp.HtmlPartExtractor;
import com.ozacc.mail.fetch.impl.sk_jp.MultipartUtility;

/**
 * MimeMessageMail륯饹
 * <p>
 * Ѵå㳰ϡΥ饹ǥå̵뤵ޤ
 * 㳰(пͤ䰸ʤ)˳Mail󥹥󥹤Υץѥƥˤϲ⥻åȤޤ
 * 
 * @since 1.2
 * @author Tomohiro Otsuka
 * @version $Id: MailConverter.java,v 1.1.2.11 2005/01/22 12:22:45 otsuka Exp $
 */
public class MailConverter {

	private static final String ATTACHMENT_DIR_PREFIX = "OML_";

	private static final String JAVA_IO_TMPDIR = "java.io.tmpdir";

	private static Log log = LogFactory.getLog(MailConverter.class);

	private static Pattern receivedHeaderPattern = Pattern.compile("^from (.+?) .*by (.+?) .*$");

	/**
	 * ¸줿źեե¸֡ǥեȤ12֡
	 */
	private long attachmentLifetime = 3600 * 1000 * 12;

	private MimeMessage[] messages;

	/**
	 * @param mimeMessage 
	 */
	public MailConverter(MimeMessage mimeMessage) {
		this(new MimeMessage[] { mimeMessage });
	}

	/**
	 * @param mimeMessages 
	 */
	public MailConverter(MimeMessage[] mimeMessages) {
		this.messages = mimeMessages;
	}

	/**
	 * @return
	 */
	public ReceivedMail[] convertIntoMails() {
		log.debug("" + messages.length + "̤MimeMessageMailѴޤ");
		List list = new ArrayList();
		for (int i = 0; i < messages.length; i++) {
			log.debug((i + 1) + "ܤMimeMessageMailѴޤ");

			MimeMessage mm = messages[i];
			ReceivedMail mail = new ReceivedMail();

			setReturnPath(mm, mail);
			setReceivedHeaders(mm, mail);
			setDate(mm, mail);
			setFromAddress(mm, mail);
			setRecipientAddresses(mm, mail);
			setMessageId(mm, mail);
			setReplyToAddress(mm, mail);
			setSubject(mm, mail);
			setXHeaders(mm, mail);
			setText(mm, mail);
			setHtmlText(mm, mail);
			setAttachmentFiles(mm, mail);

			setSize(mm, mail);

			mail.setMessage(mm);

			log.debug((i + 1) + "ܤMimeMessageMailѴޤ");
			log.debug(mail.toString());

			list.add(mail);
		}
		log.debug("" + messages.length + "̤MimeMessageMailѴޤ");
		return (ReceivedMail[])list.toArray(new ReceivedMail[list.size()]);
	}

	/**
	 * @param mm
	 * @param mail 
	 */
	private void setReceivedHeaders(MimeMessage mm, ReceivedMail mail) {
		String[] headerValues = null;
		try {
			headerValues = mm.getHeader("Received");
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		if (headerValues != null) {
			for (int i = 0; i < headerValues.length; i++) {
				String received = headerValues[i];
				// from ǻϤޤΤФԤ
				if (received.startsWith("from")) {
					received = received.replaceAll("\n", "").replaceAll("\\s+", " ");
					log.debug("Received='" + received + "'");

					Matcher m = receivedHeaderPattern.matcher(received);
					if (m.matches()) {
						String from = m.group(1);
						String by = m.group(2);
						log.debug("Sent from '" + from + "', Received by '" + by + "'");
						ReceivedHeader rh = new ReceivedHeader(from, by);
						mail.addReceviedHeader(rh);
					}
				}
			}

		}
	}

	/**
	 * ꤵ줿MimeMessageźդƤեФ
	 * ƥץѥƥ˥åȤ줿եǥ쥯ȥ¸塢
	 * Υեꤵ줿ReceivedMail˥åȤޤ
	 * <p>
	 * ¸줿źեեJVMλ˺ޤ
	 * 
	 * @param mm
	 * @param mail 
	 */
	private void setAttachmentFiles(MimeMessage mm, ReceivedMail mail) {
		try {
			cleanTempDir();

			AttachmentsExtractor ae = new AttachmentsExtractor(
					AttachmentsExtractor.MODE_IGNORE_MESSAGE
							| AttachmentsExtractor.MODE_IGNORE_INLINE);
			MultipartUtility.process(mm, ae);
			for (int i = 0, num = ae.getCount(); i < num; i++) {
				String fileName = ae.getFileName(i);
				if (fileName == null || "".equals(fileName)) {
					fileName = "attachment" + (i + 1) + ".tmp";
				}
				String path = getTempDirPath() + File.separator + ATTACHMENT_DIR_PREFIX
						+ System.currentTimeMillis() + File.separator + fileName;
				log.debug((i + 1) + "ܤźեե¸ޤ[" + path + "]");
				File f = new File(path);
				f.getParentFile().mkdirs();
				InputStream is = ae.getInputStream(i);
				writeTo(f, is);

				f.getParentFile().deleteOnExit();
				f.deleteOnExit();

				mail.addFile(f, fileName);
				log.debug((i + 1) + "ܤźեե¸ޤ[" + path + "]");
			}
		} catch (IOException e) {
			log.error("źեեμ˼Ԥޤ", e);
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
	}

	/**
	 * ǥ쥯ȥ¸줿źեե⡢¸֤ۤƤΤޤ
	 */
	private void cleanTempDir() {
		File tempDir = new File(getTempDirPath());
		File[] omlDirs = tempDir.listFiles(new FilenameFilter() {

			public boolean accept(File dir, String name) {
				return name.startsWith(ATTACHMENT_DIR_PREFIX);
			}
		});
		log.debug("" + omlDirs.length + "Ĥźեեѥǥ쥯ȥ꤬ǥ쥯ȥ¸ߤޤ");
		long now = System.currentTimeMillis();
		for (int i = 0; i < omlDirs.length; i++) {
			File dir = omlDirs[i];
			log.debug(dir.lastModified() + "");
			if (now - dir.lastModified() >= attachmentLifetime) {
				deleteDir(dir);
			}
		}
	}

	/**
	 * ǥ쥯ȥΥѥ֤ޤ
	 * 
	 * @return ǥ쥯ȥΥѥ
	 */
	private String getTempDirPath() {
		return System.getProperty(JAVA_IO_TMPDIR);
	}

	/**
	 * ꤵ줿ǥ쥯ȥȤΥեޤƺޤ
	 * 
	 * @param dir ǥ쥯ȥ
	 */
	private void deleteDir(File dir) {
		File[] files = dir.listFiles();
		for (int i = 0; i < files.length; i++) {
			File f = files[i];
			f.delete();
		}
		dir.delete();
	}

	/**
	 * ꤵ줿InputStreamǡꤵ줿ե¸ޤ
	 * 
	 * @param destFile ¸ե
	 * @param is ȤʤInputStream
	 * @throws FileNotFoundException
	 * @throws IOException 
	 */
	private void writeTo(File destFile, InputStream is) throws FileNotFoundException, IOException {
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(destFile));
		byte[] b = new byte[8192];
		while (is.read(b) != -1) {
			bos.write(b);
		}
		bos.flush();
		bos.close();
		is.close();
	}

	/**
	 * ꤵ줿MimeMessageX-HeaderReferencesIn-Reply-ToإåϤ
	 * ꤵ줿ReceivedMail˥åȤޤ
	 * 
	 * @param mm
	 * @param mail
	 */
	private void setXHeaders(MimeMessage mm, ReceivedMail mail) {
		log.debug("X-HeaderMail˥åȤޤ");
		String[] xHeaders = null;
		Enumeration headerEnum = null;
		try {
			headerEnum = mm.getAllHeaders();
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		while (headerEnum != null && headerEnum.hasMoreElements()) {
			Header header = (Header)headerEnum.nextElement();
			if (header.getName().startsWith("X-")
					|| "References".equalsIgnoreCase(header.getName())
					|| "In-Reply-To".equalsIgnoreCase(header.getName())) {
				mail.addHeader(header.getName(), header.getValue());
				log.debug(header.getName() + "Mail˥åȤޤ[" + header.getName() + "='"
						+ header.getValue() + "']");
			}
		}
	}

	private void setReplyToAddress(MimeMessage mm, ReceivedMail mail) {
		log.debug("Reply-Toɥ쥹Mail˥åȤޤ");
		Address[] addresses = null;
		try {
			addresses = mm.getReplyTo();
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		if (addresses != null) {
			log.debug(addresses.length + "ĤReply-Toɥ쥹ĤޤǽΥɥ쥹Τ߼ޤ");
			for (int j = 0; j < addresses.length; j++) {
				Address address = addresses[j];
				mail.setReplyTo((InternetAddress)address);
				break;
			}
		} else {
			log.debug("Reply-Toɥ쥹ϸĤޤǤ");
		}
	}

	/**
	 * ᡼(byte)MimeMessageReceivedMail˥åȤޤ
	 * ˼Ԥ -1 򥻥åȤޤ
	 * 
	 * @param mm
	 * @param mail 
	 */
	private void setSize(MimeMessage mm, ReceivedMail mail) {
		try {
			mail.setSize(mm.getSize());
		} catch (MessagingException e) {
			mail.setSize(-1);
		}
	}

	/**
	 * @param mm
	 * @param mail
	 * @throws MessagingException 
	 */
	private void setHtmlText(MimeMessage mm, ReceivedMail mail) {
		try {
			HtmlPartExtractor hpe = new HtmlPartExtractor();
			MultipartUtility.process(mm, hpe);
			String htmlText = hpe.getHtml();
			mail.setHtmlText(htmlText);
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
	}

	private void setText(MimeMessage mm, ReceivedMail mail) {
		try {
			String text = MultipartUtility.getPlainText(mm);
			mail.setText(text);
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
	}

	private void setMessageId(MimeMessage mm, ReceivedMail mail) {
		try {
			String messageId = mm.getMessageID();
			mail.setMessageId(messageId);
			log.debug("Message-IDMail˥åȤޤ[Message-ID='" + messageId + "']");
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
	}

	private void setSubject(MimeMessage mm, ReceivedMail mail) {
		try {
			String subject = mm.getSubject();
			mail.setSubject(subject);
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
	}

	private void setDate(MimeMessage mm, ReceivedMail mail) {
		try {
			Date d = mm.getSentDate();
			mail.setDate(d);
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
	}

	/**
	 * Return-Pathɥ쥹ɬ⥻åȤƤϤޤ
	 * äspamϤΥ᡼ǤʥեޥåȤΥ᡼륢ɥ쥹
	 * åȤƤ⤢Τա
	 * 
	 * @param mm
	 * @param mail
	 */
	private void setReturnPath(MimeMessage mm, ReceivedMail mail) {
		log.debug("Return-Pathɥ쥹򸡽Фޤ");
		String[] returnPath = null;
		try {
			returnPath = mm.getHeader("Return-Path");
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		if (returnPath != null && returnPath.length > 0) {
			String email = returnPath[0].substring(1, returnPath[0].length() - 1);
			if (email.length() > 0) {
				try {
					mail.setReturnPath(email);
					log.debug("Return-Pathɥ쥹Mail˥åȤޤ[Return-Path='" + email + "']");
				} catch (IllegalArgumentException e) {
					log.warn("Return-Pathɥ쥹ʥ᡼륢ɥ쥹եޥåȤǤ[Return-Path='" + email + "']");
				}
			} else {
				log.debug("Return-Pathɥ쥹ϸĤޤǤ");
			}
		} else {
			log.debug("Return-Pathɥ쥹ϸĤޤǤ");
		}
	}

	private void setFromAddress(MimeMessage mm, ReceivedMail mail) {
		log.debug("Fromɥ쥹򸡽Фޤ");
		Address[] addresses = null;
		try {
			addresses = mm.getFrom();
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		if (addresses != null) {
			log.debug(addresses.length + "ĤFromɥ쥹Ĥޤ");
			for (int j = 0; j < addresses.length; j++) {
				InternetAddress address = (InternetAddress)addresses[j];
				mail.setFrom(address);
				log.debug("Fromɥ쥹Mail˥åȤޤ[From='" + address.toUnicodeString() + "']");
			}
		} else {
			log.debug("Fromɥ쥹ϸĤޤǤ");
		}
	}

	private void setRecipientAddresses(MimeMessage mm, ReceivedMail mail) {
		/*
		 * TOɥ쥹Υѡ
		 */
		log.debug("Toɥ쥹򸡽Фޤ");
		Address[] toAddresses = null;
		try {
			toAddresses = mm.getRecipients(Message.RecipientType.TO);
		} catch (AddressException e) {
			log.warn("ʥ᡼륢ɥ쥹Фޤ[" + e.getRef() + "]");
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		if (toAddresses != null) {
			log.debug(toAddresses.length + "ĤToɥ쥹Ĥޤ");
			for (int j = 0; j < toAddresses.length; j++) {
				InternetAddress address = (InternetAddress)toAddresses[j];
				mail.addTo(address);
				log.debug("Toɥ쥹Mail˥åȤޤ[To='" + address.toUnicodeString() + "']");
			}
		} else {
			log.debug("Toɥ쥹ϸĤޤǤ");
		}

		/*
		 * CCɥ쥹Υѡ
		 */
		log.debug("Ccɥ쥹򸡽Фޤ");
		Address[] ccAddresses = null;
		try {
			ccAddresses = mm.getRecipients(Message.RecipientType.CC);
		} catch (AddressException e) {
			log.warn("ʥ᡼륢ɥ쥹Фޤ[" + e.getRef() + "]");
		} catch (MessagingException e) {
			// ignore
			log.warn(e.getMessage());
		}
		if (ccAddresses != null) {
			log.debug(ccAddresses.length + "ĤCcɥ쥹Ĥޤ");
			for (int j = 0; j < ccAddresses.length; j++) {
				InternetAddress address = (InternetAddress)ccAddresses[j];
				mail.addCc(address);
				log.debug("Ccɥ쥹Mail˥åȤޤ[Cc='" + address.toUnicodeString() + "']");
			}
		} else {
			log.debug("Ccɥ쥹ϸĤޤǤ");
		}
	}

}